@@ -1,6 +1,9 @@
 """(A down payment on) Testing for ``xonsh.base_shell.BaseShell`` and associated classes"""
+
 import os
 
+import pytest
+
 from xonsh.base_shell import BaseShell
 from xonsh.shell import transform_command
 
@@ -36,3 +39,28 @@ def test_transform(xession):
     assert transform_command("spam") == "egg"
     assert transform_command("egg") == "egg"
     assert transform_command("foo") == "foo"
+
+
+@pytest.mark.parametrize(
+    "cmd,exp_append_history",
+    [
+        ("", False),
+        ("# a comment", False),
+        ("print('yes')", True),
+    ],
+)
+def test_default_append_history(cmd, exp_append_history, xonsh_session, monkeypatch):
+    """Test that running an empty line or a comment does not append to history"""
+    append_history_calls = []
+
+    def mock_append_history(**info):
+        append_history_calls.append(info)
+
+    monkeypatch.setattr(
+        xonsh_session.shell.shell, "_append_history", mock_append_history
+    )
+    xonsh_session.shell.default(cmd)
+    if exp_append_history:
+        assert len(append_history_calls) == 1
+    else:
+        assert len(append_history_calls) == 0
