@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 """Tests the xonsh lexer."""
-from __future__ import unicode_literals, print_function
+
 import os
 import sys
 from collections.abc import Sequence
@@ -10,10 +9,8 @@ from pprint import pformat
 
 import pytest
 
-from xonsh.ply.ply.lex import LexToken
-
-
 from xonsh.lexer import Lexer
+from xonsh.ply.ply.lex import LexToken
 
 LEXER_ARGS = {"lextab": "lexer_test_table", "debug": 0}
 
@@ -28,7 +25,7 @@ def ensure_tuple(x):
     elif isinstance(x, Sequence):
         x = tuple(x)
     else:
-        raise TypeError("{0} is not a sequence".format(x))
+        raise TypeError(f"{x} is not a sequence")
     return x
 
 
@@ -42,7 +39,7 @@ def tokens_equal(x, y):
 def assert_token_equal(x, y):
     """Asserts that two tokens are equal."""
     if not tokens_equal(x, y):
-        msg = "The tokens differ: {0!r} != {1!r}".format(x, y)
+        msg = f"The tokens differ: {x!r} != {y!r}"
         pytest.fail(msg)
     return True
 
@@ -63,10 +60,14 @@ def assert_tokens_equal(x, y):
     return True
 
 
+def lex_input(inp: str):
+    lex = Lexer()
+    lex.input(inp)
+    return list(lex)
+
+
 def check_token(inp, exp):
-    l = Lexer()
-    l.input(inp)
-    obs = list(l)
+    obs = lex_input(inp)
     if len(obs) != 1:
         msg = "The observed sequence does not have length-1: {0!r} != 1\n"
         msg += "# obs\n{1}"
@@ -75,16 +76,12 @@ def check_token(inp, exp):
 
 
 def check_tokens(inp, exp):
-    l = Lexer()
-    l.input(inp)
-    obs = list(l)
+    obs = lex_input(inp)
     return assert_tokens_equal(exp, obs)
 
 
 def check_tokens_subproc(inp, exp, stop=-1):
-    l = Lexer()
-    l.input("$[{}]".format(inp))
-    obs = list(l)[1:stop]
+    obs = lex_input(f"$[{inp}]")[1:stop]
     return assert_tokens_equal(exp, obs)
 
 
@@ -395,7 +392,7 @@ def test_path_fstring_literal():
 def test_regex_globs():
     for i in (".*", r"\d*", ".*#{1,2}"):
         for p in ("", "r", "g", "@somethingelse", "p", "pg"):
-            c = "{}`{}`".format(p, i)
+            c = f"{p}`{i}`"
             assert check_token(c, ["SEARCHPATH", c, 0])
 
 
@@ -427,10 +424,8 @@ def test_ioredir(case):
 
 @pytest.mark.parametrize("case", [">", ">>", "<", "e>", "> ", ">>   ", "<  ", "e> "])
 def test_redir_whitespace(case):
-    inp = "![{}/path/to/file]".format(case)
-    l = Lexer()
-    l.input(inp)
-    obs = list(l)
+    inp = f"![{case}/path/to/file]"
+    obs = lex_input(inp)
     assert obs[2].type == "WS"
 
 
