@@ -1,118 +1,108 @@
-# -*- coding: utf-8 -*-
 """Test XonshLexer for pygments"""
 
-import gc
-
 import pytest
 from pygments.token import (
-    Keyword,
-    Name,
-    String,
     Error,
+    Keyword,
+    Literal,
+    Name,
     Number,
     Operator,
     Punctuation,
+    String,
     Text,
-    Literal,
 )
-from tools import skip_if_on_windows
 
-from xonsh.platform import ON_WINDOWS
-from xonsh.built_ins import XSH
-from xonsh.pyghooks import XonshLexer, Color, XonshStyle, on_lscolors_change
 from xonsh.environ import LsColors
-from xonsh.events import events, EventManager
-from tools import DummyShell
+from xonsh.events import EventManager, events
+from xonsh.pyghooks import Color, XonshLexer, XonshStyle, on_lscolors_change
+from xonsh.pytest.tools import DummyShell, skip_if_on_windows
 
 
-@pytest.fixture(autouse=True)
-def load_command_cache(xession):
-    gc.collect()
-    XSH.unload()
-    XSH.load()
-    if ON_WINDOWS:
-        for key in ("cd", "bash"):
-            xession.aliases[key] = lambda *args, **kwargs: None
+@pytest.fixture
+def xsh(xession, monkeypatch):
+    for key in ("cd", "bash"):
+        monkeypatch.setitem(xession.aliases, key, lambda *args, **kwargs: None)
 
 
-def check_token(code, tokens):
-    """Make sure that all tokens appears in code in order"""
-    lx = XonshLexer()
-    tks = list(lx.get_tokens(code))
+@pytest.fixture()
+def check_token(xsh):
+    def factory(code, tokens):
+        """Make sure that all tokens appears in code in order"""
+        lx = XonshLexer()
+        tks = list(lx.get_tokens(code))
 
-    for tk in tokens:
-        while tks:
-            if tk == tks[0]:
+        for tk in tokens:
+            while tks:
+                if tk == tks[0]:
+                    break
+                tks = tks[1:]
+            else:
+                msg = f"Token {tk!r} missing: {list(lx.get_tokens(code))!r}"
+                pytest.fail(msg)
                 break
-            tks = tks[1:]
-        else:
-            msg = "Token {!r} missing: {!r}".format(tk, list(lx.get_tokens(code)))
-            pytest.fail(msg)
-            break
+
+    return factory
 
 
-@skip_if_on_windows
-def test_ls():
-    check_token("ls -al", [(Name.Builtin, "ls")])
-
-
-@skip_if_on_windows
-def test_bin_ls():
-    check_token("/bin/ls -al", [(Name.Builtin, "/bin/ls")])
-
-
-@skip_if_on_windows
-def test_py_print():
-    check_token(
-        'print("hello")',
-        [
+_cases = {
+    "ls": {
+        "ls -al": [
+            (Name.Builtin, "ls"),
+        ],
+    },
+    "ls-bin": {
+        "/bin/ls -al": [
+            (Name.Builtin, "/bin/ls"),
+        ],
+    },
+    "print": {
+        'print("hello")': [
             (Name.Builtin, "print"),
             (Punctuation, "("),
             (Literal.String.Double, '"'),
             (Literal.String.Double, "hello"),
             (Literal.String.Double, '"'),
             (Punctuation, ")"),
-            (Text, "\n"),
+            (Text.Whitespace, "\n"),
+        ]
+    },
+    "invalid-cmd": {
+        "non-existance-cmd -al": [
+            (Name, "non"),
         ],
-    )
-
-
-@skip_if_on_windows
-def test_invalid_cmd():
-    check_token("non-existance-cmd -al", [(Name, "non")])  # parse as python
-    check_token(
-        "![non-existance-cmd -al]", [(Error, "non-existance-cmd")]
-    )  # parse as error
-    check_token("for i in range(10):", [(Keyword, "for")])  # as py keyword
-    check_token("(1, )", [(Punctuation, "("), (Number.Integer, "1")])
-
-
-@skip_if_on_windows
-def test_multi_cmd():
-    check_token(
-        "cd && cd", [(Name.Builtin, "cd"), (Operator, "&&"), (Name.Builtin, "cd")]
-    )
-    check_token(
-        "cd || non-existance-cmd",
-        [(Name.Builtin, "cd"), (Operator, "||"), (Error, "non-existance-cmd")],
-    )
-
-
-@skip_if_on_windows
-def test_nested():
-    check_token(
-        'echo @("hello")',
-        [
+        "![non-existance-cmd -al]": [
+            (Error, "non-existance-cmd"),
+        ],
+        "for i in range(10):": [
+            (Keyword, "for"),
+        ],
+        "(1, )": [
+            (Punctuation, "("),
+            (Number.Integer, "1"),
+        ],
+    },
+    "multi-cmd": {
+        "cd && cd": [
+            (Name.Builtin, "cd"),
+            (Operator, "&&"),
+            (Name.Builtin, "cd"),
+        ],
+        "cd || non-existance-cmd": [
+            (Name.Builtin, "cd"),
+            (Operator, "||"),
+            (Error, "non-existance-cmd"),
+        ],
+    },
+    "nested": {
+        'echo @("hello")': [
             (Name.Builtin, "echo"),
             (Keyword, "@"),
             (Punctuation, "("),
             (String.Double, "hello"),
             (Punctuation, ")"),
         ],
-    )
-    check_token(
-        "print($(cd))",
-        [
+        "print($(cd))": [
             (Name.Builtin, "print"),
             (Punctuation, "("),
             (Keyword, "$"),
@@ -120,12 +110,9 @@ def test_nested():
             (Name.Builtin, "cd"),
             (Punctuation, ")"),
             (Punctuation, ")"),
-            (Text, "\n"),
+            (Text.Whitespace, "\n"),
         ],
-    )
-    check_token(
-        r'print(![echo "])\""])',
-        [
+        r'print(![echo "])\""])': [
             (Name.Builtin, "print"),
             (Punctuation, "("),
             (Keyword, "!"),
@@ -135,9 +122,55 @@ def test_nested():
             (Literal.String.Double, '"])\\""'),
             (Punctuation, "]"),
             (Punctuation, ")"),
-            (Text, "\n"),
+            (Text.Whitespace, "\n"),
         ],
-    )
+    },
+    "subproc-args": {
+        "cd 192.168.0.1": [
+            (Text, "192.168.0.1"),
+        ],
+    },
+    "backtick": {
+        r"echo g`.*\w+`": [
+            (String.Affix, "g"),
+            (String.Backtick, "`"),
+            (String.Regex, "."),
+            (String.Regex, "*"),
+            (String.Escape, r"\w"),
+        ],
+    },
+    "macro": {
+        r"g!(42, *, 65)": [
+            (Name, "g"),
+            (Keyword, "!"),
+            (Punctuation, "("),
+            (Number.Integer, "42"),
+        ],
+        r"echo! hello world": [
+            (Name.Builtin, "echo"),
+            (Keyword, "!"),
+            (String, "hello world"),
+        ],
+        r"bash -c ! export var=42; echo $var": [
+            (Name.Builtin, "bash"),
+            (Text, "-c"),
+            (Keyword, "!"),
+            (String, "export var=42; echo $var"),
+        ],
+    },
+}
+
+
+def _convert_cases():
+    for title, input_dict in _cases.items():
+        for idx, item in enumerate(input_dict.items()):
+            yield pytest.param(*item, id=f"{title}-{idx}")
+
+
+@pytest.mark.parametrize("inp, expected", list(_convert_cases()))
+@skip_if_on_windows
+def test_xonsh_lexer(inp, expected, check_token):
+    check_token(inp, expected)
 
 
 # can't seem to get thie test to import pyghooks and define on_lscolors_change handler like live code does.
@@ -161,24 +194,21 @@ def xonsh_builtins_ls_colors(xession, events_fxt):
 
 
 @skip_if_on_windows
-def test_path(tmpdir, xonsh_builtins_ls_colors):
-
+def test_path(tmpdir, xonsh_builtins_ls_colors, check_token):
     test_dir = str(tmpdir.mkdir("xonsh-test-highlight-path"))
+    check_token(f"cd {test_dir}", [(Name.Builtin, "cd"), (Color.BOLD_BLUE, test_dir)])
     check_token(
-        "cd {}".format(test_dir), [(Name.Builtin, "cd"), (Color.BOLD_BLUE, test_dir)]
+        f"cd {test_dir}-xxx",
+        [(Name.Builtin, "cd"), (Text, f"{test_dir}-xxx")],
     )
-    check_token(
-        "cd {}-xxx".format(test_dir),
-        [(Name.Builtin, "cd"), (Text, "{}-xxx".format(test_dir))],
-    )
-    check_token("cd X={}".format(test_dir), [(Color.BOLD_BLUE, test_dir)])
+    check_token(f"cd X={test_dir}", [(Color.BOLD_BLUE, test_dir)])
 
     with xonsh_builtins_ls_colors.env.swap(AUTO_CD=True):
         check_token(test_dir, [(Name.Constant, test_dir)])
 
 
 @skip_if_on_windows
-def test_color_on_lscolors_change(tmpdir, xonsh_builtins_ls_colors):
+def test_color_on_lscolors_change(tmpdir, xonsh_builtins_ls_colors, check_token):
     """Verify colorizer returns Token.Text if file type not defined in LS_COLORS"""
 
     lsc = xonsh_builtins_ls_colors.env["LS_COLORS"]
@@ -186,50 +216,8 @@ def test_color_on_lscolors_change(tmpdir, xonsh_builtins_ls_colors):
 
     lsc["di"] = ("GREEN",)
 
-    check_token(
-        "cd {}".format(test_dir), [(Name.Builtin, "cd"), (Color.GREEN, test_dir)]
-    )
+    check_token(f"cd {test_dir}", [(Name.Builtin, "cd"), (Color.GREEN, test_dir)])
 
     del lsc["di"]
 
-    check_token("cd {}".format(test_dir), [(Name.Builtin, "cd"), (Text, test_dir)])
-
-
-@skip_if_on_windows
-def test_subproc_args():
-    check_token("cd 192.168.0.1", [(Text, "192.168.0.1")])
-
-
-@skip_if_on_windows
-def test_backtick():
-    check_token(
-        r"echo g`.*\w+`",
-        [
-            (String.Affix, "g"),
-            (String.Backtick, "`"),
-            (String.Regex, "."),
-            (String.Regex, "*"),
-            (String.Escape, r"\w"),
-        ],
-    )
-
-
-@skip_if_on_windows
-def test_macro():
-    check_token(
-        r"g!(42, *, 65)",
-        [(Name, "g"), (Keyword, "!"), (Punctuation, "("), (Number.Integer, "42")],
-    )
-    check_token(
-        r"echo! hello world",
-        [(Name.Builtin, "echo"), (Keyword, "!"), (String, "hello world")],
-    )
-    check_token(
-        r"bash -c ! export var=42; echo $var",
-        [
-            (Name.Builtin, "bash"),
-            (Text, "-c"),
-            (Keyword, "!"),
-            (String, "export var=42; echo $var"),
-        ],
-    )
+    check_token(f"cd {test_dir}", [(Name.Builtin, "cd"), (Text, test_dir)])
