@@ -1,42 +1,47 @@
-# -*- coding: utf-8 -*-
 """Import hooks for importing xonsh source files.
 
 This module registers the hooks it defines when it is imported.
 """
+
+import contextlib
+import importlib
 import os
 import re
 import sys
 import types
-import builtins
-import contextlib
-import importlib
+from importlib.abc import Loader, MetaPathFinder, SourceLoader
 from importlib.machinery import ModuleSpec
-from importlib.abc import MetaPathFinder, SourceLoader, Loader
 
+from xonsh.built_ins import XSH
 from xonsh.events import events
 from xonsh.execer import Execer
-from xonsh.platform import scandir
 from xonsh.lazyasd import lazyobject
+from xonsh.platform import ON_WINDOWS
+from xonsh.tools import print_warning
 
 
 @lazyobject
 def ENCODING_LINE():
     # this regex comes from PEP 263
     # https://www.python.org/dev/peps/pep-0263/#defining-the-encoding
-    return re.compile(b"^[ tv]*#.*?coding[:=][ t]*([-_.a-zA-Z0-9]+)")
+    return re.compile(b"^[ \t\f]*#.*?coding[:=][ \t]*([-_.a-zA-Z0-9]+)")
 
 
 def find_source_encoding(src):
-    """Finds the source encoding given bytes representing a file. If
-    no encoding is found, UTF-8 will be returned as per the docs
+    """Finds the source encoding given bytes representing a file by checking
+    a special comment at either the first or second line of the source file.
     https://docs.python.org/3/howto/unicode.html#unicode-literals-in-python-source-code
+    If no encoding is found, UTF-8 codec with BOM signature will be returned
+    as it skips an optional UTF-8 encoded BOM at the start of the data
+    and is otherwise the same as UTF-8
+    https://docs.python.org/3/library/codecs.html#module-encodings.utf_8_sig
     """
-    utf8 = 'UTF-8'
-    first, _, rest = src.partition(b'\n')
+    utf8 = "utf-8-sig"
+    first, _, rest = src.partition(b"\n")
     m = ENCODING_LINE.match(first)
     if m is not None:
         return m.group(1).decode(utf8)
-    second, _, _ = rest.partition(b'\n')
+    second, _, _ = rest.partition(b"\n")
     m = ENCODING_LINE.match(second)
     if m is not None:
         return m.group(1).decode(utf8)
@@ -46,44 +51,32 @@ def find_source_encoding(src):
 class XonshImportHook(MetaPathFinder, SourceLoader):
     """Implements the import hook for xonsh source files."""
 
-    def __init__(self, *args, **kwargs):
-        super(XonshImportHook, self).__init__(*args, **kwargs)
+    def __init__(self, execer, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         self._filenames = {}
-        self._execer = None
-
-    @property
-    def execer(self):
-        if hasattr(builtins, '__xonsh_execer__'):
-            execer = builtins.__xonsh_execer__
-            if self._execer is not None:
-                self._execer = None
-        elif self._execer is None:
-            self._execer = execer = Execer(unload=False)
-        else:
-            execer = self._execer
-        return execer
+        self._execer = execer
 
     #
     # MetaPathFinder methods
     #
     def find_spec(self, fullname, path, target=None):
         """Finds the spec for a xonsh module if it exists."""
-        dot = '.'
+        dot = "."
         spec = None
         path = sys.path if path is None else path
         if dot not in fullname and dot not in path:
             path = [dot] + path
         name = fullname.rsplit(dot, 1)[-1]
-        fname = name + '.xsh'
+        fname = name + ".xsh"
         for p in path:
             if not isinstance(p, str):
                 continue
             if not os.path.isdir(p) or not os.access(p, os.R_OK):
                 continue
-            if fname not in (x.name for x in scandir(p)):
+            if fname not in {x.name for x in os.scandir(p)}:
                 continue
             spec = ModuleSpec(fullname, self)
-            self._filenames[fullname] = os.path.join(p, fname)
+            self._filenames[fullname] = os.path.abspath(os.path.join(p, fname))
             break
         return spec
 
@@ -95,7 +88,7 @@ class XonshImportHook(MetaPathFinder, SourceLoader):
         mod = types.ModuleType(spec.name)
         mod.__file__ = self.get_filename(spec.name)
         mod.__loader__ = self
-        mod.__package__ = spec.parent or ''
+        mod.__package__ = spec.parent or ""
         return mod
 
     def get_filename(self, fullname):
@@ -104,30 +97,41 @@ class XonshImportHook(MetaPathFinder, SourceLoader):
 
     def get_data(self, path):
         """Gets the bytes for a path."""
-        raise NotImplementedError
+        raise OSError
 
     def get_code(self, fullname):
         """Gets the code object for a xonsh file."""
         filename = self.get_filename(fullname)
         if filename is None:
-            msg = "xonsh file {0!r} could not be found".format(fullname)
+            msg = f"xonsh file {fullname!r} could not be found"
             raise ImportError(msg)
-        with open(filename, 'rb') as f:
-            src = f.read()
-        enc = find_source_encoding(src)
-        src = src.decode(encoding=enc)
-        src = src if src.endswith('\n') else src + '\n'
-        execer = self.execer
+        src = self.get_source(fullname)
+        execer = self._execer
         execer.filename = filename
         ctx = {}  # dummy for modules
         code = execer.compile(src, glbs=ctx, locs=ctx)
         return code
 
+    def get_source(self, fullname):
+        if fullname is None:
+            raise ImportError("could not find fullname to module")
+        filename = self.get_filename(fullname)
+        with open(filename, "rb") as f:
+            src = f.read()
+        if ON_WINDOWS:
+            src = src.replace(b"\r\n", b"\n")
+        enc = find_source_encoding(src)
+        src = src.decode(encoding=enc)
+        src = src if src.endswith("\n") else src + "\n"
+        return src
+
 
 #
 # Import events
 #
-events.doc('on_import_pre_find_spec', """
+events.doc(
+    "on_import_pre_find_spec",
+    """
 on_import_pre_find_spec(fullname: str, path: str, target: module or None) -> None
 
 Fires before any import find_spec() calls have been executed. The parameters
@@ -137,9 +141,12 @@ here are the same as importlib.abc.MetaPathFinder.find_spec(). Namely,
 :``path``: None if a top-level import, otherwise the ``__path__`` of the parent
           package.
 :``target``: Target module used to make a better guess about the package spec.
-""")
+""",
+)
 
-events.doc('on_import_post_find_spec', """
+events.doc(
+    "on_import_post_find_spec",
+    """
 on_import_post_find_spec(spec, fullname, path, target) -> None
 
 Fires after all import find_spec() calls have been executed. The parameters
@@ -150,44 +157,59 @@ here the spec and the arguments importlib.abc.MetaPathFinder.find_spec(). Namely
 :``path``: None if a top-level import, otherwise the ``__path__`` of the parent
           package.
 :``target``: Target module used to make a better guess about the package spec.
-""")
+""",
+)
 
-events.doc('on_import_pre_create_module', """
+events.doc(
+    "on_import_pre_create_module",
+    """
 on_import_pre_create_module(spec: ModuleSpec) -> None
 
 Fires right before a module is created by its loader. The only parameter
 is the spec object. See importlib for more details.
-""")
+""",
+)
 
-events.doc('on_import_post_create_module', """
+events.doc(
+    "on_import_post_create_module",
+    """
 on_import_post_create_module(module: Module, spec: ModuleSpec) -> None
 
 Fires after a module is created by its loader but before the loader returns it.
 The parameters here are the module object itself and the spec object.
 See importlib for more details.
-""")
+""",
+)
 
-events.doc('on_import_pre_exec_module', """
+events.doc(
+    "on_import_pre_exec_module",
+    """
 on_import_pre_exec_module(module: Module) -> None
 
 Fires right before a module is executed by its loader. The only parameter
 is the module itself. See importlib for more details.
-""")
+""",
+)
 
-events.doc('on_import_post_exec_module', """
+events.doc(
+    "on_import_post_exec_module",
+    """
 on_import_post_create_module(module: Module) -> None
 
 Fires after a module is executed by its loader but before the loader returns it.
 The only parameter is the module itself. See importlib for more details.
-""")
+""",
+)
 
 
 def _should_dispatch_xonsh_import_event_loader():
     """Figures out if we should dispatch to a load event"""
-    return (len(events.on_import_pre_create_module) > 0 or
-            len(events.on_import_post_create_module) > 0 or
-            len(events.on_import_pre_exec_module) > 0 or
-            len(events.on_import_post_exec_module) > 0)
+    return (
+        len(events.on_import_pre_create_module) > 0
+        or len(events.on_import_post_create_module) > 0
+        or len(events.on_import_pre_exec_module) > 0
+        or len(events.on_import_post_exec_module) > 0
+    )
 
 
 class XonshImportEventHook(MetaPathFinder):
@@ -218,8 +240,9 @@ class XonshImportEventHook(MetaPathFinder):
         npost = len(events.on_import_post_find_spec)
         dispatch_load = _should_dispatch_xonsh_import_event_loader()
         if npre > 0:
-            events.on_import_pre_find_spec.fire(fullname=fullname, path=path,
-                                                target=target)
+            events.on_import_pre_find_spec.fire(
+                fullname=fullname, path=path, target=target
+            )
         elif npost == 0 and not dispatch_load:
             # no events to fire, proceed normally and prevent recursion
             return None
@@ -228,13 +251,34 @@ class XonshImportEventHook(MetaPathFinder):
             spec = importlib.util.find_spec(fullname)
         # fire post event
         if npost > 0:
-            events.on_import_post_find_spec.fire(spec=spec, fullname=fullname,
-                                                 path=path, target=target)
-        if dispatch_load and spec is not None and hasattr(spec.loader, 'create_module'):
+            events.on_import_post_find_spec.fire(
+                spec=spec, fullname=fullname, path=path, target=target
+            )
+        if dispatch_load and spec is not None and hasattr(spec.loader, "create_module"):
             spec.loader = XonshImportEventLoader(spec.loader)
         return spec
 
 
+_XIEVL_WRAPPED_ATTRIBUTES = frozenset(
+    [
+        "load_module",
+        "module_repr",
+        "get_data",
+        "get_resource_filename",
+        "get_resource_stream",
+        "get_resource_string",
+        "has_resource",
+        "has_metadata",
+        "get_metadata",
+        "get_metadata_lines",
+        "resource_isdir",
+        "metadata_isdir",
+        "resource_listdir",
+        "metadata_listdir",
+    ]
+)
+
+
 class XonshImportEventLoader(Loader):
     """A class that dispatches loader calls to another loader and fires relevant
     xonsh events.
@@ -260,16 +304,16 @@ class XonshImportEventLoader(Loader):
         events.on_import_post_exec_module.fire(module=module)
         return rtn
 
-    def load_module(self, fullname):
-        """Legacy module loading, provided for backwards compatibility."""
-        return self.loader.load_module(fullname)
-
-    def module_repr(self, module):
-        """Legacy module repr, provided for backwards compatibility."""
-        return self.loader.module_repr(module)
+    def __getattr__(self, name):
+        if name in _XIEVL_WRAPPED_ATTRIBUTES:
+            return getattr(self.loader, name)
+        return object.__getattribute__(self, name)
 
 
-def install_import_hooks():
+ARG_NOT_PRESENT = object()
+
+
+def install_import_hooks(execer=ARG_NOT_PRESENT):
     """
     Install Xonsh import hooks in ``sys.meta_path`` in order for ``.xsh`` files
     to be importable and import events to be fired.
@@ -277,6 +321,16 @@ def install_import_hooks():
     Can safely be called many times, will be no-op if xonsh import hooks are
     already present.
     """
+    if execer is ARG_NOT_PRESENT:
+        print_warning(
+            "No execer was passed to install_import_hooks. "
+            "This will become an error in future."
+        )
+        execer = XSH.execer
+        if execer is None:
+            execer = Execer()
+            XSH.load(execer=execer)
+
     found_imp = found_event = False
     for hook in sys.meta_path:
         if isinstance(hook, XonshImportHook):
@@ -284,7 +338,7 @@ def install_import_hooks():
         elif isinstance(hook, XonshImportEventHook):
             found_event = True
     if not found_imp:
-        sys.meta_path.append(XonshImportHook())
+        sys.meta_path.append(XonshImportHook(execer))
     if not found_event:
         sys.meta_path.insert(0, XonshImportEventHook())
 
