@@ -1,13 +1,17 @@
 """Lazy imports that may apply across the xonsh package."""
+
 import importlib
+import os
 
-from xonsh.platform import ON_WINDOWS, ON_DARWIN
 from xonsh.lazyasd import LazyObject, lazyobject
+from xonsh.platform import ON_DARWIN, ON_WINDOWS
 
-pygments = LazyObject(lambda: importlib.import_module('pygments'),
-                      globals(), 'pygments')
-pyghooks = LazyObject(lambda: importlib.import_module('xonsh.pyghooks'),
-                      globals(), 'pyghooks')
+pygments = LazyObject(
+    lambda: importlib.import_module("pygments"), globals(), "pygments"
+)
+pyghooks = LazyObject(
+    lambda: importlib.import_module("xonsh.pyghooks"), globals(), "pyghooks"
+)
 
 
 @lazyobject
@@ -15,7 +19,7 @@ def pty():
     if ON_WINDOWS:
         return
     else:
-        return importlib.import_module('pty')
+        return importlib.import_module("pty")
 
 
 @lazyobject
@@ -23,7 +27,7 @@ def termios():
     if ON_WINDOWS:
         return
     else:
-        return importlib.import_module('termios')
+        return importlib.import_module("termios")
 
 
 @lazyobject
@@ -31,7 +35,7 @@ def fcntl():
     if ON_WINDOWS:
         return
     else:
-        return importlib.import_module('fcntl')
+        return importlib.import_module("fcntl")
 
 
 @lazyobject
@@ -39,7 +43,7 @@ def tty():
     if ON_WINDOWS:
         return
     else:
-        return importlib.import_module('tty')
+        return importlib.import_module("tty")
 
 
 @lazyobject
@@ -80,4 +84,17 @@ def macutils():
 
 @lazyobject
 def terminal256():
-    return importlib.import_module('pygments.formatters.terminal256')
+    return importlib.import_module("pygments.formatters.terminal256")
+
+
+@lazyobject
+def html():
+    return importlib.import_module("pygments.formatters.html")
+
+
+@lazyobject
+def os_listxattr():
+    def dummy_listxattr(*args, **kwargs):
+        return []
+
+    return getattr(os, "listxattr", dummy_listxattr)
